#pragma once

#include <PxPhysicsAPI.h>
using namespace physx; // For now will just focus on getting Physx working in its most basic form

#define PVD_HOST "127.0.0.1"
#define MAX_IP_CHAR_COUNT 16

namespace YoggieEngine {

	struct PhysicsConfig {
		
		bool recordMemoryAllocations;
		const char PvdHost[MAX_IP_CHAR_COUNT];
		float StackZ;

	};

	class Physics  {
	public:

		Physics();
		~Physics();

		void Step(float dt);

		void Demo();

		void EnableDebugVisuals();
		void SubmitMesh();

		void DebugRender(Framebuffer& framebuffer);


	private:
		Shader debugDraw = Shader("build/Debug/Shaders/forward/debug.vert", "build/Debug/Shaders/forward/debug.frag");

		PxRigidDynamic* createDynamic(const PxTransform& t, const PxGeometry& geometry, const PxVec3& velocity);
		void createStack(const PxTransform& t, PxU32 size, PxReal halfextent);
		void createScene();
		void createGroundPlane();

		void SetupPvdDebug();


		
		// Memory Management
		bool recordMemoryAllocations = true;
		PxDefaultErrorCallback gDefaultErrorCallback;
		PxDefaultAllocator gDefaultAllocatorCallback;
		PxDefaultCpuDispatcher* gDispatcher;

		// Basics 
		PxFoundation* mFoundation;
		PxPhysics* mPhysics;

		// Simulation
		PxScene* mScene;
		PxMaterial* gMaterial;

		PxReal stackZ = 10.0f;

		// DEBUG 
		PxPvd* mPvd;
		PxPvdTransport* transport;
		
	};

}




